/* single object view header */
/* TODO relates to potentially redundant relations_detail_generic.html template */
#single-object-header h2:nth-of-type(2) {
    text-align: center;
}

/* navigation between objects (in list) from single object view */
/* TODO relates to potentially redundant relations_detail_generic.html template */
.next-in-list {
    float: right;
}
