# Generated by Django 4.2.9 on 2024-01-26 07:57

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("collections", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="skoscollection",
            options={"ordering": ["name"]},
        ),
    ]
