# Generated by Django 4.2.8 on 2023-12-22 07:56

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("apis_vocabularies", "0003_delete_vocabnames_delete_vocabsuri"),
    ]

    operations = [
        migrations.DeleteModel(
            name="CollectionType",
        ),
    ]
