{% extends "apis_core/apis_entities/abstractentity.html" %}
{% load crispy_forms_tags %}
{% load static %}

{% block col-zero %}
  <div class="card">
    <div class="card-body">{% crispy form form.helper %}</div>
    <div class="card-footer">{% include "apis_entities/partials/linked_open_data.html" %}</div>
  </div>
{% endblock col-zero %}

{% block scripts %}
  {{ block.super }}
  <script src="{% static "js/apis_entities.js" %}"></script>
{% endblock scripts %}
