{# djlint:off #}
<a href="#"
   class="reledit text-warning"
   title="edit"
   onclick=GetFormAjax(null,"{% if record.relation_pk %}{{ record.relation_pk }}{% else %}{{ record.pk }}{% endif %}")>
{# djlint:on #}
<span class="material-symbols-outlined">edit</span>
</a>
