{% extends "base.html" %}

{% block relations-menu-items %}
  {{ block.super }}
  <a class="dropdown-item"
     href="{% url 'apis_core:apis_relations:generic_relations_list' "property" %}">Properties</a>
  <a class="dropdown-item"
     href="{% url 'apis_core:apis_relations:generic_relations_list' "triple" %}">Triples</a>
{% endblock relations-menu-items %}
