{% load generic %}
<div class="card">
  <div class="card-header">{{ object }}</div>
  {% template_list object "_card_table.html" as template_list %}
  <div class="card-body">{% include template_list %}</div>
</div>
