# Generated by Django 4.2.10 on 2024-02-26 13:20

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("apis_relations", "0004_auto_20230124_1201"),
    ]

    operations = [
        migrations.AlterField(
            model_name="property",
            name="obj_class",
            field=models.ManyToManyField(
                related_name="property_set_obj", to="contenttypes.contenttype"
            ),
        ),
        migrations.AlterField(
            model_name="property",
            name="subj_class",
            field=models.ManyToManyField(
                related_name="property_set_subj", to="contenttypes.contenttype"
            ),
        ),
    ]
