# Generated by Django 4.2.8 on 2023-12-22 07:56

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("apis_vocabularies", "0004_delete_collectiontype"),
    ]

    operations = [
        migrations.DeleteModel(
            name="VocabsBaseClass",
        ),
    ]
