# Generated by Django 3.1.14 on 2022-02-01 22:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("apis_relations", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="property",
            name="property_class_uri",
            field=models.CharField(
                blank=True, max_length=255, verbose_name="Property Class URI"
            ),
        ),
    ]
