<a title="delete"
   hx-delete="{{ record.get_api_detail_endpoint }}"
   hx-confirm="Are your sure you want to delete {{ record }}?"
   hx-target="closest tr"
   hx-swap="outerHTML swap:0.3s"
   href="{{ record.get_delete_url }}"
   class="text-danger"><span class="material-symbols-outlined">delete</span></a>
