# Generated by Django 4.2.8 on 2023-12-22 07:55

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("apis_vocabularies", "0002_remove_texttype_collections_and_more"),
    ]

    operations = [
        migrations.DeleteModel(
            name="VocabNames",
        ),
        migrations.DeleteModel(
            name="VocabsUri",
        ),
    ]
