{% load relations %}
{% relations_from from_obj=object as relations %}
{% possible_relation_types_from object as possible_relations %}
{% get_relation_targets_from object as possible_targets %}

{% block display-relations %}
  {% for target in possible_targets %}
    <div class="card mb-2">
      <div class="card-header">{{ target.name }}</div>
      <div class="card-body">{% include "relations/list_relations.html" %}</div>
    </div>
  {% endfor %}
{% endblock display-relations %}
