{% extends basetemplate|default:"base.html" %}

{% block content %}
  <div class="container">
    <div class="text-center">
      <form action="" method="post">
        {% csrf_token %}
        <h4>Confirm deletion of:</h4>
        <h3>
          <strong>{{ object }}</strong>
        </h3>
        <input class="btn btn-danger" type="submit" value="Yes, delete" />
      </form>
    </div>
  </div>
{% endblock content %}
