{% load core %}
{% with object|opts as opts %}
  {% if object.get_listview_url %}
    <a href="{{ object.get_listview_url }}"><small>{{ opts.verbose_name_plural }}</small></a>
  {% else %}
    <small>{{ opts.verbose_name_plural }}</small>
  {% endif %}
{% endwith %}
