# Generated by Django 4.2.8 on 2023-12-22 07:29

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("apis_metainfo", "0008_alter_rootobject_self_contenttype"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="collection",
            name="collection_type",
        ),
    ]
