{% extends "apis_core/apis_entities/abstractentity_form.html" %}
{% load static %}

{% block styles %}
  {{ block.super }}
  <link href="{% static 'css/relations.css' %}" rel="stylesheet" />
{% endblock styles %}

{% block content %}
  {{ block.super }}
  <dialog id="relationdialog">
    <div id="relation-dialog-content"></div>
  </dialog>
{% endblock content %}

{% block col-one %}

  {% block relations-include %}
    {% include "relations/list_relations_include.html" with edit=True %}
  {% endblock relations-include %}

  {{ block.super }}
{% endblock col-one %}

{% block scripts %}
  {{ block.super }}
  <script src="{% static "js/relation_dialog.js" %}"></script>
{% endblock scripts %}
